<?php

declare(strict_types=1);

namespace Internetgalerie\IgFrontendUser\Domain\Repository;

use Internetgalerie\IgFrontendUser\Event\AddFrontendUserConstraintsEvent;
use Psr\EventDispatcher\EventDispatcherInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Persistence\Generic\Typo3QuerySettings;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * A Frontend User repository
 */
class FrontendUserRepository extends Repository
{
    private EventDispatcherInterface $eventDispatcher;
    private ConfigurationManager $configurationManager;
    
    public function injectEventDispatcher(EventDispatcherInterface $eventDispatcher): void
    {
        $this->eventDispatcher = $eventDispatcher;
    }

    public function injectConfigurationManager(ConfigurationManager $configurationManager): void
    {
        $this->configurationManager = $configurationManager;
        $querySettings = GeneralUtility::makeInstance(Typo3QuerySettings::class);
        //$querySettings->setRespectStoragePage(false);
        $storagePid = (int) ($this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT)['plugin.']['tx_igfrontenduser.']['persistence.']['storagePid'] ?? 0);
        $querySettings->setStoragePageIds([$storagePid]);
        $this->setDefaultQuerySettings($querySettings);
 
    }
    public function findAllFromStoragePid()
    {
        $query = $this->createQuery();
        $this->configurationManager = GeneralUtility::makeInstance(ConfigurationManager::class);
        $storagePid = (int) ($this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT)['plugin.']['tx_igfrontenduser.']['persistence.']['storagePid'] ?? 0);
        $query->getQuerySettings()->setStoragePageIds([$storagePid]);
        return $query->execute();
    }
          
    public function findWithUsergroups(array $usergroupUids, bool $returnRawQueryResult = false)
    {
        $query = $this->createQuery();

        $constraints = [];
        foreach ($usergroupUids as $usergroupUid) {
            // FIND_IN_SET
            $constraints[] = $query->logicalOr([
                $query->equals('usergroup', (int)$usergroupUid),
                $query->like('usergroup', (int)$usergroupUid . ',%'),
                $query->like('usergroup', '%,' . (int)$usergroupUid),
                $query->like('usergroup', '%,' . (int)$usergroupUid . ',%'),
            ]);
        }
        if (!empty($constraints)) {
            $query->matching(
                $query->logicalAnd($constraints)
            );
        }
        return $query->execute($returnRawQueryResult);

    }

    public function findBySearch(array $search, array $settings = [], bool $returnRawQueryResult = false) {
        $query = $this->createQuery();

        $constraints = [];
        $constraints  = $this->eventDispatcher->dispatch(
            new AddFrontendUserConstraintsEvent($query, $search, $settings)
        )->getConstraints();
        // Andere Sucher
        //$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('fe_users');
        //$queryBuilder->expr()->inSet('usergroup', '12');//$query->equals('usergroup', $fegroup);
        $constraints  = $this->eventDispatcher->dispatch(
            new AddFrontendUserConstraintsEvent($query, $search, $settings)
        )->getConstraints();

        if($settings['forceFrontendUserGroupUid']) {
            $fegroup = (int)$settings['forceFrontendUserGroupUid'];
            $feGroupConstrain = $query->logicalOr([
                $query->equals('usergroup', (int)$fegroup),
                $query->like('usergroup', (int)$fegroup . ',%'),
                $query->like('usergroup', '%,' . (int)$fegroup),
                $query->like('usergroup', '%,' . (int)$fegroup . ',%'),
            ]);
            $constraints[] = $feGroupConstrain;
        }

        if (isset($search['fegroup']) && $search['fegroup'] !== null && $search['fegroup'] !== '') {
            // @todo find_in_set + explode
            $fegroup = (int) $search['fegroup'];
            $feGroupHasNot = $fegroup < 0;
            $fegroup = abs($fegroup);
            // fegroup
            $feGroupConstrain = $query->logicalOr([
                $query->equals('usergroup', (int)$fegroup),
                $query->like('usergroup', (int)$fegroup . ',%'),
                $query->like('usergroup', '%,' . (int)$fegroup),
                $query->like('usergroup', '%,' . (int)$fegroup . ',%'),
            ]);
            if ($feGroupHasNot) {
                $constraints[] =  $query->logicalNot($feGroupConstrain);
            } else {
                $constraints[] = $feGroupConstrain;
            }
        }

        $keywordsArray = GeneralUtility::trimExplode(' ', $search['keywords'], true);
        if (!empty($keywordsArray)) {
            foreach ($keywordsArray as $keyword) {
                $constraints[] = $query->logicalOr(
                    $query->like('username', '%' . $keyword . '%'),
                    $query->like('first_name', '%' . $keyword . '%'),
                    $query->like('last_name', '%' . $keyword . '%')
                );
            }
        }
        if (!empty($constraints)) {
            $query->matching(
                $query->logicalAnd($constraints)
            );
        }
        return $query->execute($returnRawQueryResult);
    }
}
